#!/usr/bin/ksh

#-----------------------------------------------------------------------------------------
#--
#-- Copyright (c) 2009-2012 ProactiveRA
#--
#-- This script is free software; you can redistribute it and/or modify it under
#-- the terms of the GNU Lesser General Public License as published by the Free
#-- Software Foundation; either version 3 of the License, or (at your option)
#-- any later version.
#--
#-- This script is distributed in the hope that it will be useful, but WITHOUT
#-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#-- FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
#-- details.
#--
#-- You should have received a copy of the GNU Lesser General Public License
#-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--
#-----------------------------------------------------------------------------------------

. ../Common/lib.ksh


#identify if we need to run in batch mode or interactive mode
if [[ $# = 0 ]];then
  running_mode=interactive
else
  running_mode=batch
fi

MODULE="BIV"
init_global_variables
SOURCE="            (Loading Script Menu)            ";

# Post Processing Procedure
oracle_proc_name=P_BIV_MAIN_LOAD

read_resource_file

#############################

function load {
 
  if [[ $running_mode = "interactive" ]];then
     clear
  fi

  start_and_prepare_load_logfile $1
  print_header $1

  # Verify availability of database before attempting loading anything
  validate_RA_DB_available

  if [ $abort = 0 ]; then
  	# No Error - Continue
	echo "  Parsing the raw data files..."
	echo "  Parsing the raw data files..." >> $logfile
	echo "  -----------------------------------------"
	echo "  -----------------------------------------" >> $logfile

	nb_file=0
        # Parse the raw files or just copy if not needed
	while read line ; do
		file_pattern=`echo $line | sed 's/\(.*\);\(.*\);\(.*\)/\1/'`
		file_parser=`echo $line | sed 's/\(.*\);\(.*\);\(.*\)/\2/'`
  		for myfile in $(ls ${ready_dir}/${main_directory[$1]}/${file_pattern} 2> /dev/null); do
     			if [[ -d  $myfile ]];then
				# This is a directory.. skip
        			unused=1
     			else
				((nb_file=nb_file+1))
				file_name=`echo $myfile | sed 's/.*\///'`
				echo "  [$file_name]"
				echo "  [$file_name]" >> $logfile
				command_line="${parsing_prg_dir}/${file_parser}"
				command_line="${command_line} $myfile"
				command_line="${command_line} ${parsed_dir}/${main_directory[$1]}/${file_name}.parsed"

				result=`${command_line}`
				echo "$result"
				echo "$result" >> $logfile

				mv $myfile ${ready_dir}/${main_directory[$1]}/processed
				echo "    => File moved to 'processed' directory"
				echo "    => File moved to 'processed' directory" >> $logfile


			fi
		done
	done < ${resource_dir}/${main_directory[$1]}/datasource.resource

	if [[ $nb_file = 0 ]];then
		echo "    <<No File to Process>>"
		echo "    <<No File to Process>>" >> $logfile
	else
	  echo "  -------------------------------------"
	  echo "  -------------------------------------" >> $logfile
	  echo "  Nb Files Processed: ${nb_file}"
	  echo "  Nb Files Processed: ${nb_file}" >> $logfile
	  echo "  -------------------------------------"
  	  echo "  -------------------------------------" >> $logfile
  	  echo "  Done."
  	  echo "  Done." >> $logfile
        fi 

	echo ""
	echo "" >> $logfile
	echo "  Loading the datasource extract files..." 
	echo "  Loading the datasource extract files..." >> $logfile
	echo "  ---------------------------------------" 
	echo "  ---------------------------------------" >> $logfile

	# Load the files in database
  	nb_file=0
	while read line ; do

		file_pattern=`echo $line | sed 's/\(.*\);\(.*\);\(.*\)/\1/'`
		control_file=`echo $line | sed 's/\(.*\);\(.*\);\(.*\)/\3/'`
		exception=0

  		for myfile in $(ls ${parsed_dir}/${main_directory[$1]}/${file_pattern}.parsed 2> /dev/null); do

     			if [[ -d  $myfile ]];then

				# This is a directory.. skip
        			unused=1
     			else
				((nb_file=nb_file+1))
				file_name=`echo $myfile | sed 's/.*\///'`
				echo "  [${file_name}]"
				echo "  [${file_name}]" >> $logfile 

                		command_line="sqlldr "
                		command_line="$command_line control=${sqlldr_home_dir}/ctl/${main_directory[$1]}/${control_file} "
                		command_line="$command_line data=${myfile} "
                		command_line="$command_line log=${sqlldr_home_dir}/log/${main_directory[$1]}/${file_name}.log "
                		command_line="$command_line bad=${sqlldr_home_dir}/bad/${main_directory[$1]}/${file_name}.bad  "
			        result=`$command_line > $sqlldrlogfile 2>&1 << ENDOFSQL
                			${UID_RA}/${PASS_RA}@${DB_RA}
ENDOFSQL
`


				# Finding the number of successfully loaded records
				command_line="grep -h successfully "
        			command_line="$command_line ${sqlldr_home_dir}/log/${main_directory[$1]}/${file_name}.log"
				result=`$command_line`
				if [ "$result" ]
				then
					echo "    => ${result}" | sed 's/>   /> /' 
					echo "    => ${result}" | sed 's/>   /> /' >> $logfile

	  				# Moving the loaded file to 'loaded' directory
					mv $myfile ${loaded_dir}/${main_directory[$1]}/${file_name}.loaded
					echo "    => File moved to 'loaded' directory"
					echo "    => File moved to 'loaded' directory" >> $logfile

				else
	  				# Couldn't find the result of the number of rows loaded=problem. Not moving the file
					echo "    => ERROR loading in database. Check the sqlldr log."
					echo "    => ERROR loading in database. Check the sqlldr log." >> $logfile

					exception=1
				fi

			fi
		done

	  done < ${resource_dir}/${main_directory[$1]}/datasource.resource
	  if [[ $nb_file = 0 ]];then
		exception=1
                echo "    <<No File to Process>>"
                echo "    <<No File to Process>>" >> $logfile
          else
          	echo "  -------------------------------------"
          	echo "  -------------------------------------" >> $logfile
          	echo "  Nb Files Processed: ${nb_file}"
          	echo "  Nb Files Processed: ${nb_file}" >> $logfile
          	echo "  -------------------------------------"
          	echo "  -------------------------------------" >> $logfile
          	echo "  Done."
          	echo "  Done." >> $logfile
          	echo "" 
          	echo "" >> $logfile
	  fi

	  if [[ $exception = 0 ]]; then

	  # Perform the Post-Loading

          # Execute the SQL-Procedures to load in the final tables

	  echo ""
	  echo "" >> $logfile
  	  echo "  Starting the Post-Processing..." 
	  echo "  Starting the Post-Processing..." >> $logfile
  	  echo "  -------------------------------------"
  	  echo "  -------------------------------------" >> $logfile

	  # Execute the post-processing
	  echo "  Executing ${oracle_proc_name}('${oracle_table[$1]}')"
	  echo "  Executing ${oracle_proc_name}('${oracle_table[$1]}')" >> $logfile
	  result=`sqlplus -s /NOLOG << ENDOFSQL
	  connect ${UID_RA}/${PASS_RA}@${DB_RA}
	  execute ${oracle_proc_name}('${oracle_table[$1]}');
	  exit;
ENDOFSQL
`
	  #look for Oracle error codes 
	  search_and_log_ORACLE_error ${DB_RA}
          if [ $abort = 0 ]; then
	        # No Error
		echo "    => Success"
		echo "    => Success" >> $logfile

	  fi 
      fi

  fi

  echo "  -------------------------------------"
  echo "  -------------------------------------" >> $logfile
  echo "  Done."
  echo "  Done." >> $logfile

  echo ""
  echo "" >> $logfile
  if [[ $running_mode = "interactive" ]];then
     echo "Press Enter to return to the main menu"
     read unused
  fi
}

#############################

choice=-1

if [[ $running_mode = "batch" ]];then
  load $1
else

  while [ $choice -ne 0 ]
  do
      display_sources
      read choice

      if [[ $choice = 0 ]]; then
	  clear;
	  exit 1;
      fi
      if [[ $choice == +([0-9]) ]]; then
          if [[ $choice -le $nbsources ]]; then
	      load $choice;
          else	
  	      clear; 
	      echo "Wrong option entered"; 
	      echo "Press Enter"; 
	      read unused; 
	      choice=-1;
          fi
      else
          clear; 
          echo "Wrong option entered"; 
          echo "Press Enter"; 
          read unused; 
          choice=-1;
      fi
  done
fi
