#!/usr/bin/ksh

#-----------------------------------------------------------------------------------------
#--
#-- Copyright (c) 2009-2012 ProactiveRA
#--
#-- This script is free software; you can redistribute it and/or modify it under
#-- the terms of the GNU Lesser General Public License as published by the Free
#-- Software Foundation; either version 3 of the License, or (at your option)
#-- any later version.
#--
#-- This script is distributed in the hope that it will be useful, but WITHOUT
#-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#-- FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
#-- details.
#--
#-- You should have received a copy of the GNU Lesser General Public License
#-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--
#-----------------------------------------------------------------------------------------

. ../Common/lib.ksh

#identify if we need to run in batch mode or interactive mode
if [[ $# = 0 ]];then
  running_mode=interactive
else
  running_mode=batch
fi

MODULE="CDR"

init_global_variables

# Available Reconciliations
Reconciliation[1]="SAMPLE SOURCE  <=> OTHER SOURCE"

# Reconciliation Parameters 
Recon_Param_A[1]="SAMPLE_SOURCE"; 
Recon_Param_B[1]="OTHER_SOURCE";
nbrecon=1

# Oracle Reconciling Procedures
oracle_proc_name_1=P_CDR_INSERT_JOB_COMPARE
oracle_proc_name_2=P_CDR_EXECUTE_JOB_COMPARE

#############################

function get_date_parameter {

clear

echo ""; echo ""; echo ""
echo "    ###############################################"
echo "    #                                             #"
echo "    #            - REVENUE ASSURANCE -            #"
echo "    #                xDR Assurance                #"
echo "    #                                             #"
echo "    #              Reconcile CDR-to-CDR           #"
echo "    #                                             #"
echo "    ###############################################"

echo ""
echo ""
echo "Enter the Date parameters for this CDR-to-CDR Reconcile job:"
echo "------------------------------------------------------------"
correct_date=0
while [ $correct_date -ne 1 ]
do
   echo "Start Date/Time (format DD/MM/YYYY HH24:MI:SS):"
   read date_min
   correct_date=1
done
correct_date=0
while [ $correct_date -ne 1 ]
do
   echo "End Date/Time (format DD/MM/YYYY HH24:MI:SS):"
   read date_max
   correct_date=1
done

echo ""
echo ""
echo "Enter the Service Type Id for this CDR-to-CDR Reconcile job:"
echo "------------------------------------------------------------"
correct_st_id=0
while [ $correct_st_id -ne 1 ]
do
   echo "Service Type Id:"
   read st_id
   correct_st_id=1
done

echo ""
echo ""
echo "Enter the Tolerance (in sec) for the date/time comparison:"
echo "----------------------------------------------------------"
correct_tolerance=0
while [ $correct_tolerance -ne 1 ]
do
   echo "Tolerance level (in sec):"
   read tolerance
   correct_tolerance=1
done

clear

}

function reconcile_option {
 
  if [[ $running_mode = "interactive" ]];then
     get_date_parameter
  fi

  # Prepare name of the log file
  date=$(date +%Y%m%d_%H%M%S)
  logfile=${logfile_dir}/reconciling_${date}.log
  touch $logfile
  abort=0

  echo "##############################"  
  echo "##############################"  >> $logfile
  echo ${Reconciliation[$1]} 
  echo ${Reconciliation[$1]} >> $logfile
  echo "##############################" 
  echo "##############################" >> $logfile
  echo "  Starting the CDR-to-CDR Reconciliation..." 
  echo "  Starting the CDR-to-CDR Reconciliation..." >> $logfile
  echo "  -----------------------------------------" 
  echo "  -----------------------------------------" >> $logfile

  # Step 1 - Verify availability of database before attempting loading anything
  validate_RA_DB_available
  if [ $abort = 0 ]; then

  	# No Error - Continue
  	echo "  Getting the next Job Id"
  	echo "  Getting the next Job Id" >> $logfile

	# Get the next Job-Id
  	result=`sqlplus -s /NOLOG << ENDOFSQL
		connect ${UID_RA}/${PASS_RA}@${DB_RA}
  		set pagesize 0
  		set linesize 230
  		set heading off
  		set feedback off
  		set underline off
  		set define off
  		select 'Job-ID=', SEQ_CDR_RECONCILIATION_JOB.NEXTVAL, '#' from dual;
  		exit;
ENDOFSQL
`
	job_id=`echo $result | sed 's/.*Job-ID= *//' | sed 's/ *#.*//'`
	echo "    => ${job_id}" 
	echo "    => ${job_id}" >> $logfile
	echo ""
	echo "" >> $logfile
	echo "  Preparing Reconciliation Job ${job_id}" 
	echo "  Preparing Reconciliation Job ${job_id}" >> $logfile

 # Step 2: Insert the job in the database
	parameters="V_JOB_ID => $job_id"
	parameters="$parameters, 	V_TABLE_A => '${Recon_Param_A[$1]}' "
	parameters="$parameters, 	V_TABLE_B => '${Recon_Param_B[$1]}' "
	parameters="$parameters,	V_TRANS_DT_MIN => TO_DATE('$date_min','DD/MM/YYYY HH24:MI:SS')"
	parameters="$parameters,	V_TRANS_DT_MAX => TO_DATE('$date_max','DD/MM/YYYY HH24:MI:SS')" 
	parameters="$parameters,	V_SERVICE_TYPE_ID => $st_id" 
	parameters="$parameters,	V_TOLERANCE => $tolerance"
	formated_params=`echo $parameters | sed 's/,  */,\\\n		/g'`
	echo "  Executing ${oracle_proc_name_1}($formated_params)"
	echo "  Executing ${oracle_proc_name_1}($formated_params)" >> $logfile
	result=`sqlplus -s /NOLOG << ENDOFSQL
	connect ${UID_RA}/${PASS_RA}@${DB_RA}
	execute ${oracle_proc_name_1}($parameters);
	exit;
ENDOFSQL
`
	#look for Oracle error codes 
	 search_and_log_ORACLE_error ${DB_RA}
         if [ $abort = 0 ]; then

 # Step 3: Execute the SQL-Procedures to compare CDR-to-CDR

	  # Execute the CDR-to-CDR procedure
	  parameters="JOB_ID => $job_id"
	  formated_params=`echo $parameters | sed 's/,  */,\\\n		/g'`
	  echo "  Executing ${oracle_proc_name_2}($formated_params)"
	  echo "  Executing ${oracle_proc_name_2}($formated_params)" >> $logfile
	  result=`sqlplus -s /NOLOG << ENDOFSQL
	  connect ${UID_RA}/${PASS_RA}@${DB_RA}
	  execute ${oracle_proc_name_2}($parameters);
	  exit;
ENDOFSQL
`
	  #look for Oracle error codes
	  search_and_log_ORACLE_error ${DB_RA}
          if [ $abort = 0 ]; then
	  	# No Error
		echo "    => Success"
		echo "    => Success" >> $logfile

          	# Get the results from the RESULT_CDR_SUMMARY table
	  	result=`sqlplus -s /NOLOG << ENDOFSQL
		connect ${UID_RA}/${PASS_RA}@${DB_RA}
		set pagesize 0
		set linesize 230
		set heading off
		set feedback off
		set underline off
		set define off
	  	SELECT '#Table_A=' || TABLE_A  || '#Table_B=' ||  TABLE_B || '#1=' || TO_CHAR(IN_A_AND_B)
			||  '#2=' || TO_CHAR(IN_A_NOT_B) || '#3=' || TO_CHAR(IN_B_NOT_A) || '#' 
		FROM RESULT_CDR_SUMMARY 
                WHERE JOB_ID=${job_id};
  	  	exit;
ENDOFSQL
`
		table_a=`echo $result | tr -d '\n' | sed 's/.*#Table_A= *//' | sed 's/ *#Table_B=.*//'`
		table_b=`echo $result | tr -d '\n' | sed 's/.*#Table_B= *//' | sed 's/ *#1=.*//'`
		in_a_and_b=`echo $result | tr -d '\n' | sed 's/.*#1= *//' | sed 's/ *#2=.*//'`
		in_a_not_b=`echo $result | tr -d '\n' | sed 's/.*#2= *//' | sed 's/ *#3=.*//'`
		in_b_not_a=`echo $result | tr -d '\n' | sed 's/.*#3= *//' | sed 's/ *#.*//'`
		echo "    => $in_a_and_b CDRs successfully matched in BOTH control points"
		echo "    => $in_a_and_b CDRs successfully matched in BOTH control points" >> $logfile
		echo "    => $in_a_not_b CDRs found in $table_a not found in $table_b"
		echo "    => $in_a_not_b CDRs found in $table_a not found in $table_b" >> $logfile
		echo "    => $in_b_not_a CDRs found in $table_b not found in $table_a" 
		echo "    => $in_b_not_a CDRs found in $table_b not found in $table_a" >> $logfile

	fi 
       fi
	
  fi
  finish_log_and_end
}

#############################

choice=-1

if [[ $running_mode = "batch" ]];then
   reconcile_option $1
else

   while [ $choice -ne 0 ]
   do

      clear

      echo ""; echo ""; echo ""
      echo "    ###############################################"
      echo "    #                                             #"
      echo "    #            - REVENUE ASSURANCE -            #"
      echo "    #                xDR Assurance                #"
      echo "    #                                             #"
      echo "    #              Reconcile CDR-to-CDR           #"
      echo "    #                                             #"
      echo "    ###############################################"
      echo ""
      echo ""
      echo "Available Reconciliations:"
      echo "--------------------------"
      reconid=1;
      while [[ $reconid -le $nbrecon ]]; do
   	     echo "  $reconid ) ${Reconciliation[${reconid}]}"
   	     ((reconid=reconid+1));
      done
      echo ""
      echo "  0) Cancel"
      echo ""
      echo "Select your option: "
      read choice

      if [[ $choice = 0 ]]; then
          clear;
          exit 1;
      fi
      if [[ $choice == +([0-9]) ]]; then
          if [[ $choice -le $nbrecon ]]; then
              reconcile_option $choice;
          else
              clear; 
              echo "Wrong option entered"; 
              echo "Press Enter"; 
              read unused; 
              choice=-1;
          fi
      else
          clear; 
          echo "Wrong option entered"; 
          echo "Press Enter"; 
          read unused; 
          choice=-1;
      fi
   done
fi
