#!/usr/bin/ksh

#-----------------------------------------------------------------------------------------
#--
#-- Copyright (c) 2009-2012 ProactiveRA
#--
#-- This script is free software; you can redistribute it and/or modify it under
#-- the terms of the GNU Lesser General Public License as published by the Free
#-- Software Foundation; either version 3 of the License, or (at your option)
#-- any later version.
#--
#-- This script is distributed in the hope that it will be useful, but WITHOUT
#-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#-- FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
#-- details.
#--
#-- You should have received a copy of the GNU Lesser General Public License
#-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--
#-----------------------------------------------------------------------------------------

. ../Common/lib.ksh

#identify if we need to run in batch mode or interactive mode
if [[ $# = 0 ]];then
  running_mode=interactive
else
  running_mode=batch
fi

MODULE="SDR"

init_global_variables
read_resource_file

#############################

function purge_loads {

echo "  Purging loads of platform: $1"
echo "  Purging loads of platform: $1" >> $logfile
## first count how many loads there are
nbloads=0;
  for myload in $(ls ${main_dir}/loads/${1}); do
     if [[ -d  ${main_dir}/loads/${1}/$myload ]];then
        # This is a directory.. 
        ((nbloads=nbloads+1));
     else
	# This is a file.. 
	# There shouldn't be any file in there. Do nothing.
        unused=1
     fi
  done
((nbloads=nbloads-5));
## Just keep 5 loads 
nbdeleted=0;
  for myload in $(ls -rt ${main_dir}/loads/${1}); do
     if [[ -d  ${main_dir}/loads/${1}/$myload ]];then
        # This is a directory.. 
        if [[ $nbdeleted -lt $nbloads ]];then
		rm -rf ${main_dir}/loads/${1}/$myload
		((nbdeleted=nbdeleted+1));
	fi
     else
	# This is a file.. 
	# There shouldn't be any file in there. Do nothing.
        unused=1
     fi
  done

echo "      => ${nbdeleted} loads deleted"
echo "      => ${nbdeleted} loads deleted" >> $logfile

}

#############################

function purge {
 
  # Prepare name of the log file
  date=$(date +%Y%m%d_%H%M%S)
  logfile=${logfile_dir}/purge_SDR_files_${date}.log
  touch $logfile

  echo "##############################"  
  echo "##############################"  >> $logfile
  echo "Purge Load Files"
  echo "Purge Load Files" >> $logfile
  echo "##############################" 
  echo "##############################" >> $logfile
  echo "  Starting the Purge..." 
  echo "  Starting the Purge..." >> $logfile
  echo "  ---------------------" 
  echo "  ---------------------" >> $logfile

  for myplatform in $(ls ${main_dir}/loads); do
     if [[ -d  ${main_dir}/loads/$myplatform ]];then
        # This is a directory.. 
        purge_loads $myplatform
     else
	# This is a file.. 
	# There shouldn't be any file in there. Do nothing.
        unused=1
     fi
  done

  purge_logfiles

  echo "  ---------------------"
  echo "  ---------------------" >> $logfile
  echo "  Done."
  echo "  Done." >> $logfile

  echo ""
  echo "" >> $logfile
  if [[ $running_mode = "interactive" ]];then
     echo "Press Enter to return to the main menu"
     read unused
  fi
}


#############################

choice=-1

if [[ $running_mode = "batch" ]];then
   purge
else

   while [ $choice -ne 0 ]
   do

	clear

	echo ""; echo ""; echo ""
	echo "    ###############################################"
	echo "    #                                             #"
	echo "    #            - REVENUE ASSURANCE -            #"
	echo "    #           Subscriber/Data Integrity         #"
	echo "    #                                             #"
	echo "    #                 Purge Files                 #"
	echo "    #                                             #"
	echo "    ###############################################"
	echo ""
	echo ""
	echo "Confirmation :"
	echo "-------------"
	echo "This will erase the files permanently"
	echo "(only the last 5 loads of each platform will be kept)"
	echo ""
	echo "  1) Confirm"
	echo "  0) Cancel"
	echo ""
	echo "Select your option: "
	read choice

	case "$choice" in
	 1 ) purge ; choice=0;;
	 0 ) clear; exit 1 ;;
	 *)  clear; echo "Wrong option entered"; echo "Press Enter"; read unused; choice=-1;;
	esac

   done

fi
